import com.squareup.sqldelight.db.SqlDriver
import com.squareup.sqldelight.sqlite.driver.JdbcSqliteDriver
import controller.*
import fr.louveauAmine.cvApi.dao.CVApiDB
import fr.louveauAmine.cvApi.dao.PROFILE
import io.javalin.Javalin
import io.javalin.apibuilder.ApiBuilder
import io.javalin.core.security.RouteRole

enum class ApiRole : RouteRole { PUBLIC, BASIC_AUTHED }

fun main() {
    
    val app = Javalin.create{
        it.accessManager { handler, ctx, permittedRoles ->
            val auth = ctx.header("Authorization").orEmpty()
            val isBasicAuthed = auth == System.getenv("LDC_AUTH")
            when {
                permittedRoles.contains(ApiRole.PUBLIC) -> handler.handle(ctx)
                permittedRoles.contains(ApiRole.BASIC_AUTHED) && isBasicAuthed -> handler.handle(ctx)
                else -> ctx.status(401).json("Unauthorized")
            }
        }
    }.start(7070)


    val driver: SqlDriver = JdbcSqliteDriver(JdbcSqliteDriver.IN_MEMORY + "cvapi.db")
    CVApiDB.Schema.create(driver)
    val database = CVApiDB(driver)
    
    val meController = MeController(database)
    val educationController = EducationYearController(database)
    val interestController = InterestController(database)
    val jobController = JobController(database)
    val languageController = LanguageController(database)
    val profileController = ProfileController(database)
    val sideProjectController = SideProjectController(database)
    val tagController = TagController(database)
    app.routes {
        ApiBuilder.path("api") {
            ApiBuilder.get("me", { ctx ->
                meController.getMe(ctx)
            }, ApiRole.PUBLIC)

            ApiBuilder.post("education", { ctx ->
                educationController.create(ctx)
            }, ApiRole.BASIC_AUTHED)
            ApiBuilder.delete("education", { ctx ->
                educationController.delete(ctx)
            }, ApiRole.BASIC_AUTHED)

            ApiBuilder.post("interest", { ctx ->
                interestController.create(ctx)
            }, ApiRole.BASIC_AUTHED)
            ApiBuilder.delete("interest", { ctx ->
                interestController.delete(ctx)
            }, ApiRole.BASIC_AUTHED)

            ApiBuilder.post("job", { ctx ->
                jobController.create(ctx)
            }, ApiRole.BASIC_AUTHED)
            ApiBuilder.delete("job", { ctx ->
                jobController.delete(ctx)
            }, ApiRole.BASIC_AUTHED)

            ApiBuilder.post("language", { ctx ->
                languageController.create(ctx)
            }, ApiRole.BASIC_AUTHED)
            ApiBuilder.delete("language", { ctx ->
                languageController.delete(ctx)
            }, ApiRole.BASIC_AUTHED)

            ApiBuilder.post("side-project", { ctx ->
                sideProjectController.create(ctx)
            }, ApiRole.BASIC_AUTHED)
            ApiBuilder.delete("side-project", { ctx ->
                sideProjectController.delete(ctx)
            }, ApiRole.BASIC_AUTHED)

            ApiBuilder.post("tag", { ctx ->
                tagController.create(ctx)
            }, ApiRole.BASIC_AUTHED)
            ApiBuilder.delete("tag", { ctx ->
                tagController.delete(ctx)
            }, ApiRole.BASIC_AUTHED)

            ApiBuilder.post("profile", { ctx ->
                profileController.create(ctx)
            }, ApiRole.BASIC_AUTHED)
            ApiBuilder.patch("profile", { ctx ->
                profileController.update(ctx)
            }, ApiRole.BASIC_AUTHED)
        }
    }
}